@@ -11,9 +11,10 @@
 #    under the License.
 
 import os
-import uuid
 
+from oslo_utils import uuidutils
 import requests
+import requests.auth
 import six
 from tempest.lib import exceptions
 
@@ -29,9 +30,73 @@ class AodhClientTest(base.ClientTestBase):
         self.aodh("help", params="alarm show")
         self.aodh("help", params="alarm update")
 
+    def test_alarm_id_or_name_scenario(self):
+        def _test(name):
+            params = "create --type event --name %s" % name
+            result = self.aodh('alarm', params=params)
+            alarm_id = self.details_multiple(result)[0]['alarm_id']
+
+            params = 'show %s' % name
+            result = self.aodh('alarm', params=params)
+            self.assertEqual(alarm_id,
+                             self.details_multiple(result)[0]['alarm_id'])
+
+            params = 'show %s' % alarm_id
+            result = self.aodh('alarm', params=params)
+            self.assertEqual(alarm_id,
+                             self.details_multiple(result)[0]['alarm_id'])
+
+            params = "update --state ok %s" % name
+            result = self.aodh('alarm', params=params)
+            self.assertEqual("ok", self.details_multiple(result)[0]['state'])
+
+            params = "update --state alarm %s" % alarm_id
+            result = self.aodh('alarm', params=params)
+            self.assertEqual("alarm",
+                             self.details_multiple(result)[0]['state'])
+
+            params = "update --name another-name %s" % name
+            result = self.aodh('alarm', params=params)
+            self.assertEqual("another-name",
+                             self.details_multiple(result)[0]['name'])
+
+            params = "update --name %s %s" % (name, alarm_id)
+            result = self.aodh('alarm', params=params)
+            self.assertEqual(name,
+                             self.details_multiple(result)[0]['name'])
+
+            # Check update with no change is allowed
+            params = "update --name %s %s" % (name, name)
+            result = self.aodh('alarm', params=params)
+            self.assertEqual(name,
+                             self.details_multiple(result)[0]['name'])
+
+            params = "update --state ok"
+            result = self.aodh('alarm', params=params,
+                               fail_ok=True, merge_stderr=True)
+            self.assertFirstLineStartsWith(
+                result.splitlines(),
+                'You need to specify one of alarm ID and alarm name(--name) '
+                'to update an alarm.')
+
+            params = "delete %s" % name
+            result = self.aodh('alarm', params=params)
+            self.assertEqual("", result)
+
+            params = "create --type event --name %s" % name
+            result = self.aodh('alarm', params=params)
+            alarm_id = self.details_multiple(result)[0]['alarm_id']
+
+            params = "delete %s" % alarm_id
+            result = self.aodh('alarm', params=params)
+            self.assertEqual("", result)
+
+        _test(uuidutils.generate_uuid())
+        _test('normal-alarm-name')
+
     def test_event_scenario(self):
 
-        PROJECT_ID = str(uuid.uuid4())
+        PROJECT_ID = uuidutils.generate_uuid()
 
         # CREATE
         result = self.aodh(u'alarm',
@@ -42,15 +107,6 @@ class AodhClientTest(base.ClientTestBase):
         self.assertEqual('ev_alarm1', alarm['name'])
         self.assertEqual('*', alarm['event_type'])
 
-        # CREATE FAIL
-        result = self.aodh(u'alarm',
-                           params=(u"create --type event --name ev_alarm1 "
-                                   "--project-id %s" % PROJECT_ID),
-                           fail_ok=True, merge_stderr=True)
-        self.assertFirstLineStartsWith(
-            result.splitlines(),
-            "Alarm with name='ev_alarm1' exists (HTTP 409)")
-
         # UPDATE IGNORE INVALID
         result = self.aodh(
             'alarm', params=("update %s --severity critical --threshold 10"
@@ -77,7 +133,7 @@ class AodhClientTest(base.ClientTestBase):
 
         # GET BY NAME
         result = self.aodh(
-            'alarm', params="show --alarm-name ev_alarm1")
+            'alarm', params="show --name ev_alarm1")
         alarm_show = self.details_multiple(result)[0]
         self.assertEqual(ALARM_ID, alarm_show["alarm_id"])
         self.assertEqual(PROJECT_ID, alarm_show["project_id"])
@@ -87,7 +143,7 @@ class AodhClientTest(base.ClientTestBase):
         # GET BY NAME AND ID ERROR
         self.assertRaises(exceptions.CommandFailed,
                           self.aodh, u'alarm',
-                          params=(u"show %s --alarm-name ev_alarm1" %
+                          params=(u"show %s --name ev_alarm1" %
                                   ALARM_ID))
 
         # LIST
@@ -128,163 +184,33 @@ class AodhClientTest(base.ClientTestBase):
         self.assertNotIn(ALARM_ID,
                          [r['alarm_id'] for r in self.parser.listing(result)])
 
-    def test_threshold_scenario(self):
-
-        PROJECT_ID = str(uuid.uuid4())
-
-        # CREATE
-        result = self.aodh(u'alarm',
-                           params=(u"create --type threshold --name alarm_th "
-                                   "-m meter_name --threshold 5 "
-                                   "--project-id %s" % PROJECT_ID))
-        alarm = self.details_multiple(result)[0]
-        ALARM_ID = alarm['alarm_id']
-        self.assertEqual('alarm_th', alarm['name'])
-        self.assertEqual('meter_name', alarm['meter_name'])
-        self.assertEqual('5.0', alarm['threshold'])
-
-        # CREATE WITH --TIME-CONSTRAINT
-        result = self.aodh(
-            u'alarm',
-            params=(u"create --type threshold --name alarm_tc "
-                    "-m meter_name --threshold 5 "
-                    "--time-constraint "
-                    "name=cons1;start='0 11 * * *';duration=300 "
-                    "--time-constraint "
-                    "name=cons2;start='0 23 * * *';duration=600 "
-                    "--project-id %s" % PROJECT_ID))
-        alarm = self.details_multiple(result)[0]
-        self.assertEqual('alarm_tc', alarm['name'])
-        self.assertEqual('meter_name', alarm['meter_name'])
-        self.assertEqual('5.0', alarm['threshold'])
-        self.assertIsNotNone(alarm['time_constraints'])
-
-        # CREATE FAIL
-        result = self.aodh(u'alarm',
-                           params=(u"create --type threshold --name alarm_th "
-                                   "-m meter_name --threshold 5 "
-                                   "--project-id %s" % PROJECT_ID),
-                           fail_ok=True, merge_stderr=True)
-        self.assertFirstLineStartsWith(
-            result.splitlines(),
-            "Alarm with name='alarm_th' exists (HTTP 409)")
-
-        # CREATE FAIL MISSING PARAM
-        self.assertRaises(exceptions.CommandFailed,
-                          self.aodh, u'alarm',
-                          params=(u"create --type threshold --name alarm_th "
-                                  "--project-id %s" % PROJECT_ID))
-
-        # UPDATE
-        result = self.aodh(
-            'alarm', params=("update %s --severity critical --threshold 10"
-                             % ALARM_ID))
-        alarm_updated = self.details_multiple(result)[0]
-        self.assertEqual(ALARM_ID, alarm_updated["alarm_id"])
-        self.assertEqual('critical', alarm_updated['severity'])
-        self.assertEqual('10.0', alarm_updated["threshold"])
-
-        # GET
-        result = self.aodh(
-            'alarm', params="show %s" % ALARM_ID)
-        alarm_show = self.details_multiple(result)[0]
-        self.assertEqual(ALARM_ID, alarm_show["alarm_id"])
-        self.assertEqual(PROJECT_ID, alarm_show["project_id"])
-        self.assertEqual('alarm_th', alarm_show['name'])
-        self.assertEqual('meter_name', alarm_show['meter_name'])
-        self.assertEqual('10.0', alarm_show['threshold'])
-
-        # GET BY NAME
-        result = self.aodh(
-            'alarm', params="show --alarm-name alarm_th")
-        alarm_show = self.details_multiple(result)[0]
-        self.assertEqual(ALARM_ID, alarm_show["alarm_id"])
-        self.assertEqual(PROJECT_ID, alarm_show["project_id"])
-        self.assertEqual('alarm_th', alarm_show['name'])
-        self.assertEqual('meter_name', alarm_show['meter_name'])
-        self.assertEqual('10.0', alarm_show['threshold'])
-
-        # GET BY NAME AND ID ERROR
-        self.assertRaises(exceptions.CommandFailed,
-                          self.aodh, u'alarm',
-                          params=(u"show %s --alarm-name alarm_th" %
-                                  ALARM_ID))
-
-        # LIST
-        result = self.aodh('alarm', params="list")
-        self.assertIn(ALARM_ID,
-                      [r['alarm_id'] for r in self.parser.listing(result)])
-        output_colums = ['alarm_id', 'type', 'name', 'state', 'severity',
-                         'enabled']
-        for alarm_list in self.parser.listing(result):
-            self.assertEqual(sorted(output_colums), sorted(alarm_list.keys()))
-            if alarm_list["alarm_id"] == ALARM_ID:
-                self.assertEqual('alarm_th', alarm_list['name'])
-
-        # LIST WITH QUERY
-        result = self.aodh('alarm',
-                           params=("list --query project_id=%s" % PROJECT_ID))
-        alarm_list = self.parser.listing(result)[0]
-        self.assertEqual(ALARM_ID, alarm_list["alarm_id"])
-        self.assertEqual('alarm_th', alarm_list['name'])
-
-        # DELETE
-        result = self.aodh('alarm', params="delete %s" % ALARM_ID)
-        self.assertEqual("", result)
-
-        # GET FAIL
-        result = self.aodh('alarm', params="show %s" % ALARM_ID,
-                           fail_ok=True, merge_stderr=True)
-        expected = "Alarm %s not found (HTTP 404)" % ALARM_ID
-        self.assertFirstLineStartsWith(result.splitlines(), expected)
-
-        # DELETE FAIL
-        result = self.aodh('alarm', params="delete %s" % ALARM_ID,
-                           fail_ok=True, merge_stderr=True)
-        self.assertFirstLineStartsWith(result.splitlines(), expected)
-
-        # LIST DOES NOT HAVE ALARM
-        result = self.aodh('alarm', params="list")
-        self.assertNotIn(ALARM_ID,
-                         [r['alarm_id'] for r in self.parser.listing(result)])
-
     def test_composite_scenario(self):
 
-        project_id = str(uuid.uuid4())
+        project_id = uuidutils.generate_uuid()
+        res_id = uuidutils.generate_uuid()
         # CREATE
-        result = self.aodh(u'alarm',
-                           params=(u'create --type composite --name calarm1 '
-                                   ' --composite-rule \'{"or":[{"threshold"'
-                                   ': 0.8,"meter_name": "cpu_util",'
-                                   '"type": "threshold"},{"and": ['
-                                   '{"threshold": 200, "meter_name": '
-                                   '"disk.iops", "type": "threshold"},'
-                                   '{"threshold": 1000,"meter_name":'
-                                   '"network.incoming.packets.rate",'
-                                   '"type": "threshold"}]}]}\' '
-                                   '--project-id %s' % project_id))
+        result = self.aodh(
+            u'alarm',
+            params=(u'create --type composite --name calarm1 --composite-rule '
+                    '\'{"or":[{"threshold": 0.8, "metric": "cpu_util", '
+                    '"type": "gnocchi_resources_threshold", "resource_type": '
+                    '"generic", "resource_id": "%s", '
+                    '"aggregation_method": "mean"},'
+                    '{"and": [{"threshold": 200, "metric": "disk.iops", '
+                    '"type": "gnocchi_resources_threshold", "resource_type": '
+                    '"generic", "resource_id": "%s", '
+                    '"aggregation_method": "mean"},'
+                    '{"threshold": 1000, "metric": "memory",'
+                    '"type": "gnocchi_resources_threshold", "resource_type": '
+                    '"generic", "resource_id": "%s", '
+                    '"aggregation_method": "mean"}]}]}\' --project-id %s' %
+                    (res_id, res_id, res_id, project_id)))
         alarm = self.details_multiple(result)[0]
         alarm_id = alarm['alarm_id']
         self.assertEqual('calarm1', alarm['name'])
         self.assertEqual('composite', alarm['type'])
         self.assertIn('composite_rule', alarm)
 
-        # CREATE FAIL
-        result = self.aodh(u'alarm',
-                           params=(u'create --type composite --name calarm1 '
-                                   ' --composite-rule \'{"or":[{"threshold"'
-                                   ': 0.8,"meter_name": "cpu_util",'
-                                   '"type": "threshold"},{"and": ['
-                                   '{"threshold": 200, "meter_name": '
-                                   '"disk.iops", "type": "threshold"},'
-                                   '{"threshold": 1000,"meter_name":'
-                                   '"network.incoming.packets.rate",'
-                                   '"type": "threshold"}]}]}\' '
-                                   '--project-id %s' % project_id),
-                           fail_ok=True, merge_stderr=True)
-        self.assertFirstLineStartsWith(
-            result.splitlines(), "Alarm with name='calarm1' exists (HTTP 409)")
-
         # CREATE FAIL MISSING PARAM
         self.assertRaises(exceptions.CommandFailed,
                           self.aodh, u'alarm',
@@ -308,7 +234,7 @@ class AodhClientTest(base.ClientTestBase):
 
         # GET BY NAME
         result = self.aodh(
-            'alarm', params="show --alarm-name calarm1")
+            'alarm', params="show --name calarm1")
         alarm_show = self.details_multiple(result)[0]
         self.assertEqual(alarm_id, alarm_show["alarm_id"])
         self.assertEqual(project_id, alarm_show["project_id"])
@@ -317,7 +243,7 @@ class AodhClientTest(base.ClientTestBase):
         # GET BY NAME AND ID ERROR
         self.assertRaises(exceptions.CommandFailed,
                           self.aodh, u'alarm',
-                          params=(u"show %s --alarm-name calarm1" %
+                          params=(u"show %s --name calarm1" %
                                   alarm_id))
 
         # LIST
@@ -358,23 +284,117 @@ class AodhClientTest(base.ClientTestBase):
         self.assertNotIn(alarm_id,
                          [r['alarm_id'] for r in self.parser.listing(result)])
 
+    def _test_alarm_create_show_query(self, create_params, expected_lines):
+
+        def test(params):
+            result = self.aodh('alarm', params=params)
+            alarm = self.details_multiple(result)[0]
+            for key, value in six.iteritems(expected_lines):
+                self.assertEqual(value, alarm[key])
+            return alarm
+
+        alarm = test(create_params)
+        params = 'show %s' % alarm['alarm_id']
+        test(params)
+        self.aodh('alarm', params='delete %s' % alarm['alarm_id'])
+
+    def test_event_alarm_create_show_query(self):
+        params = ('create --type event --name alarm-multiple-query '
+                  '--query "traits.project_id=789;traits.resource_id=012"')
+        expected_lines = {
+            'query': 'traits.project_id = 789 AND',
+            '': 'traits.resource_id = 012',
+        }
+        self._test_alarm_create_show_query(params, expected_lines)
+
+        params = ('create --type event --name alarm-single-query '
+                  '--query "traits.project_id=789"')
+        expected_lines = {'query': 'traits.project_id = 789'}
+        self._test_alarm_create_show_query(params, expected_lines)
+
+        params = 'create --type event --name alarm-no-query'
+        self._test_alarm_create_show_query(params, {'query': ''})
+
+    def test_set_get_alarm_state(self):
+        result = self.aodh(
+            'alarm',
+            params=('create --type event --name alarm_state_test '
+                    '--query "traits.project_id=789;traits.resource_id=012"'))
+        alarm = self.details_multiple(result)[0]
+        alarm_id = alarm['alarm_id']
+        result = self.aodh(
+            'alarm', params="show %s" % alarm_id)
+        alarm_show = self.details_multiple(result)[0]
+        self.assertEqual('insufficient data', alarm_show['state'])
+        result = self.aodh('alarm', params="state get %s" % alarm_id)
+        state_get = self.details_multiple(result)[0]
+        self.assertEqual('insufficient data', state_get['state'])
+        self.aodh('alarm',
+                  params="state set --state ok  %s" % alarm_id)
+        result = self.aodh('alarm', params="state get %s" % alarm_id)
+        state_get = self.details_multiple(result)[0]
+        self.assertEqual('ok', state_get['state'])
+        self.aodh('alarm', params='delete %s' % alarm_id)
+
+    def test_update_type_event_composite(self):
+
+        res_id = uuidutils.generate_uuid()
+        # CREATE
+        result = self.aodh(u'alarm',
+                           params=(u"create --type event --name ev_alarm123"))
+        alarm = self.details_multiple(result)[0]
+        ALARM_ID = alarm['alarm_id']
+        self.assertEqual('ev_alarm123', alarm['name'])
+        self.assertEqual('*', alarm['event_type'])
+
+        # UPDATE TYPE TO COMPOSITE
+        result = self.aodh(
+            'alarm',
+            params=('update %s --type composite --composite-rule '
+                    '\'{"or":[{"threshold": 0.8, "metric": "cpu_util", '
+                    '"type": "gnocchi_resources_threshold", "resource_type": '
+                    '"generic", "resource_id": "%s", '
+                    '"aggregation_method": "mean"},'
+                    '{"and": [{"threshold": 200, "metric": "disk.iops", '
+                    '"type": "gnocchi_resources_threshold", "resource_type": '
+                    '"generic", "resource_id": "%s", '
+                    '"aggregation_method": "mean"},'
+                    '{"threshold": 1000, "metric": "memory",'
+                    '"type": "gnocchi_resources_threshold", "resource_type": '
+                    '"generic", "resource_id": "%s", '
+                    '"aggregation_method": "mean"}]}]}\'' %
+                    (ALARM_ID, res_id, res_id, res_id)))
+        alarm_updated = self.details_multiple(result)[0]
+        self.assertEqual(ALARM_ID, alarm_updated["alarm_id"])
+        self.assertEqual('composite', alarm_updated['type'])
+        self.assertIn('composite_rule', alarm_updated)
+
+        # UPDATE TYPE TO EVENT
+        result = self.aodh(
+            'alarm', params=("update %s --type event"
+                             % ALARM_ID))
+        alarm_updated = self.details_multiple(result)[0]
+        self.assertEqual(ALARM_ID, alarm_updated["alarm_id"])
+        self.assertEqual('event', alarm_updated['type'])
+        self.assertEqual('*', alarm_updated['event_type'])
+
+        # DELETE
+        result = self.aodh('alarm', params="delete %s" % ALARM_ID)
+        self.assertEqual("", result)
+
 
 class AodhClientGnocchiRulesTest(base.ClientTestBase):
 
     def test_gnocchi_resources_threshold_scenario(self):
 
-        PROJECT_ID = str(uuid.uuid4())
-        RESOURCE_ID = str(uuid.uuid4())
+        PROJECT_ID = uuidutils.generate_uuid()
+        RESOURCE_ID = uuidutils.generate_uuid()
 
         req = requests.post(
-            os.environ.get("GNOCCHI_ENDPOINT") + "/v1/resource/instance",
+            os.environ.get("GNOCCHI_ENDPOINT") + "/v1/resource/generic",
+            auth=requests.auth.HTTPBasicAuth('admin', ''),
             json={
-                "display_name": "myvm",
-                "flavor_id": "2", "host": "blah",
                 "id": RESOURCE_ID,
-                "image_ref": "http://image",
-                "project_id": "BD3A1E52-1C62-44CB-BF04-660BD88CD74D",
-                "user_id": "BD3A1E52-1C62-44CB-BF04-660BD88CD74D",
             })
         self.assertEqual(201, req.status_code)
 
@@ -384,7 +404,7 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
                                    "--type gnocchi_resources_threshold "
                                    "--name alarm_gn1 --metric cpu_util "
                                    "--threshold 80 "
-                                   "--resource-id %s --resource-type instance "
+                                   "--resource-id %s --resource-type generic "
                                    "--aggregation-method last "
                                    "--project-id %s"
                                    % (RESOURCE_ID, PROJECT_ID)))
@@ -396,22 +416,24 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
         self.assertEqual('last', alarm['aggregation_method'])
         self.assertEqual(RESOURCE_ID,
                          alarm['resource_id'])
-        self.assertEqual('instance', alarm['resource_type'])
+        self.assertEqual('generic', alarm['resource_type'])
 
-        # CREATE FAIL
-        result = self.aodh(u'alarm',
-                           params=(u"create "
-                                   "--type gnocchi_resources_threshold "
-                                   "--name alarm_gn1 --metric cpu_util "
-                                   "--threshold 80 "
-                                   "--resource-id %s --resource-type instance "
-                                   "--aggregation-method last "
-                                   "--project-id %s"
-                                   % (RESOURCE_ID, PROJECT_ID)),
-                           fail_ok=True, merge_stderr=True)
-        self.assertFirstLineStartsWith(
-            result.splitlines(),
-            "Alarm with name='alarm_gn1' exists (HTTP 409)")
+        # CREATE WITH --TIME-CONSTRAINT
+        result = self.aodh(
+            u'alarm',
+            params=(u"create --type gnocchi_resources_threshold "
+                    "--name alarm_tc --metric cpu_util --threshold 80 "
+                    "--resource-id %s --resource-type generic "
+                    "--aggregation-method last --project-id %s "
+                    "--time-constraint "
+                    "name=cons1;start='0 11 * * *';duration=300 "
+                    "--time-constraint "
+                    "name=cons2;start='0 23 * * *';duration=600 "
+                    % (RESOURCE_ID, PROJECT_ID)))
+        alarm = self.details_multiple(result)[0]
+        self.assertEqual('alarm_tc', alarm['name'])
+        self.assertEqual('80.0', alarm['threshold'])
+        self.assertIsNotNone(alarm['time_constraints'])
 
         # CREATE FAIL MISSING PARAM
         self.assertRaises(exceptions.CommandFailed,
@@ -419,7 +441,7 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
                           params=(u"create "
                                   "--type gnocchi_resources_threshold "
                                   "--name alarm1 --metric cpu_util "
-                                  "--resource-id %s --resource-type instance "
+                                  "--resource-id %s --resource-type generic "
                                   "--aggregation-method last "
                                   "--project-id %s"
                                   % (RESOURCE_ID, PROJECT_ID)))
@@ -444,11 +466,12 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
         self.assertEqual('90.0', alarm_show['threshold'])
         self.assertEqual('critical', alarm_show['severity'])
         self.assertEqual('last', alarm_show['aggregation_method'])
-        self.assertEqual('instance', alarm_show['resource_type'])
+        self.assertEqual('generic', alarm_show['resource_type'])
 
         # GET BY NAME
         result = self.aodh(
-            'alarm', params="show --alarm-name alarm_gn1")
+            'alarm', params="show --name alarm_gn1")
+        alarm_show = self.details_multiple(result)[0]
         self.assertEqual(ALARM_ID, alarm_show["alarm_id"])
         self.assertEqual(PROJECT_ID, alarm_show["project_id"])
         self.assertEqual('alarm_gn1', alarm_show['name'])
@@ -456,12 +479,12 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
         self.assertEqual('90.0', alarm_show['threshold'])
         self.assertEqual('critical', alarm_show['severity'])
         self.assertEqual('last', alarm_show['aggregation_method'])
-        self.assertEqual('instance', alarm_show['resource_type'])
+        self.assertEqual('generic', alarm_show['resource_type'])
 
         # GET BY NAME AND ID ERROR
         self.assertRaises(exceptions.CommandFailed,
                           self.aodh, u'alarm',
-                          params=(u"show %s --alarm-name alarm_gn1" %
+                          params=(u"show %s --name alarm_gn1" %
                                   ALARM_ID))
 
         # LIST
@@ -475,6 +498,49 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
             if alarm_list["alarm_id"] == ALARM_ID:
                 self.assertEqual('alarm_gn1', alarm_list['name'])
 
+        # LIST WITH PAGINATION
+        # list with limit
+        result = self.aodh('alarm',
+                           params="list --limit 1")
+        alarm_list = self.parser.listing(result)
+        self.assertEqual(1, len(alarm_list))
+        # list with sort with key=name dir=asc
+        result = self.aodh('alarm',
+                           params="list --sort name:asc")
+        names = [r['name'] for r in self.parser.listing(result)]
+        sorted_name = sorted(names)
+        self.assertEqual(sorted_name, names)
+        # list with sort with key=name dir=asc and key=alarm_id dir=asc
+        result = self.aodh(
+            u'alarm',
+            params=(u"create --type gnocchi_resources_threshold "
+                    "--name alarm_th --metric cpu_util --threshold 80 "
+                    "--resource-id %s --resource-type generic "
+                    "--aggregation-method last --project-id %s "
+                    % (RESOURCE_ID, PROJECT_ID)))
+        created_alarm_id = self.details_multiple(result)[0]['alarm_id']
+        result = self.aodh('alarm',
+                           params="list --sort name:asc --sort alarm_id:asc")
+        alarm_list = self.parser.listing(result)
+        ids_with_same_name = []
+        names = []
+        for alarm in alarm_list:
+            names.append(['alarm_name'])
+            if alarm['name'] == 'alarm_th':
+                ids_with_same_name.append(alarm['alarm_id'])
+        sorted_ids = sorted(ids_with_same_name)
+        sorted_names = sorted(names)
+        self.assertEqual(sorted_names, names)
+        self.assertEqual(sorted_ids, ids_with_same_name)
+        # list with sort with key=name dir=desc and with the marker equal to
+        # the alarm_id of the alarm_th we created for this test.
+        result = self.aodh('alarm',
+                           params="list --sort name:desc "
+                                  "--marker %s" % created_alarm_id)
+        self.assertIn('alarm_tc',
+                      [r['name'] for r in self.parser.listing(result)])
+        self.aodh('alarm', params="delete %s" % created_alarm_id)
+
         # LIST WITH QUERY
         result = self.aodh('alarm',
                            params=("list --query project_id=%s" % PROJECT_ID))
@@ -504,7 +570,7 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
 
     def test_gnocchi_aggr_by_resources_scenario(self):
 
-        PROJECT_ID = str(uuid.uuid4())
+        PROJECT_ID = uuidutils.generate_uuid()
 
         # CREATE
         result = self.aodh(
@@ -514,8 +580,8 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
                     "gnocchi_aggregation_by_resources_threshold "
                     "--name alarm1 --metric cpu --threshold 80 "
                     "--query "
-                    '\'{"=": {"server_group": "my_group"}}\' '
-                    "--resource-type instance "
+                    '\'{"=": {"creator": "cr3at0r"}}\' '
+                    "--resource-type generic "
                     "--aggregation-method last "
                     "--project-id %s" % PROJECT_ID))
         alarm = self.details_multiple(result)[0]
@@ -524,26 +590,10 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
         self.assertEqual('cpu', alarm['metric'])
         self.assertEqual('80.0', alarm['threshold'])
         self.assertEqual('last', alarm['aggregation_method'])
-        self.assertEqual('instance', alarm['resource_type'])
-        self.assertEqual('{"=": {"server_group": "my_group"}}',
+        self.assertEqual('generic', alarm['resource_type'])
+        self.assertEqual('{"=": {"creator": "cr3at0r"}}',
                          alarm['query'])
 
-        # CREATE FAIL
-        result = self.aodh(
-            u'alarm',
-            params=(u"create "
-                    "--type "
-                    "gnocchi_aggregation_by_resources_threshold "
-                    "--name alarm1 --metric cpu --threshold 80 "
-                    "--query "
-                    '\'{"=": {"server_group": "my_group"}}\' '
-                    "--resource-type instance "
-                    "--aggregation-method last "
-                    "--project-id %s" % PROJECT_ID),
-            fail_ok=True, merge_stderr=True)
-        self.assertFirstLineStartsWith(
-            result.splitlines(), "Alarm with name='alarm1' exists (HTTP 409)")
-
         # CREATE FAIL MISSING PARAM
         self.assertRaises(
             exceptions.CommandFailed,
@@ -553,8 +603,8 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
                     "gnocchi_aggregation_by_resources_threshold "
                     "--name alarm1 --metric cpu "
                     "--query "
-                    '\'{"=": {"server_group": "my_group"}}\' '
-                    "--resource-type instance "
+                    '\'{"=": {"creator": "cr3at0r"}}\' '
+                    "--resource-type generic "
                     "--aggregation-method last "
                     "--project-id %s" % PROJECT_ID))
 
@@ -578,7 +628,7 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
         self.assertEqual('90.0', alarm_show['threshold'])
         self.assertEqual('critical', alarm_show['severity'])
         self.assertEqual('last', alarm_show['aggregation_method'])
-        self.assertEqual('instance', alarm_show['resource_type'])
+        self.assertEqual('generic', alarm_show['resource_type'])
 
         # LIST
         result = self.aodh('alarm', params="list")
@@ -613,7 +663,7 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
 
     def test_gnocchi_aggr_by_metrics_scenario(self):
 
-        PROJECT_ID = str(uuid.uuid4())
+        PROJECT_ID = uuidutils.generate_uuid()
         METRIC1 = 'cpu'
         METRIC2 = 'cpu_util'
 
@@ -624,7 +674,7 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
                     "--type gnocchi_aggregation_by_metrics_threshold "
                     "--name alarm1 "
                     "--metrics %s "
-                    "--metrics %s "
+                    "--metric %s "
                     "--threshold 80 "
                     "--aggregation-method last "
                     "--project-id %s"
@@ -637,22 +687,6 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
         self.assertEqual('80.0', alarm['threshold'])
         self.assertEqual('last', alarm['aggregation_method'])
 
-        # CREATE FAIL
-        result = self.aodh(
-            u'alarm',
-            params=(u"create "
-                    "--type gnocchi_aggregation_by_metrics_threshold "
-                    "--name alarm1 "
-                    "--metrics %s "
-                    "--metrics %s "
-                    "--threshold 80 "
-                    "--aggregation-method last "
-                    "--project-id %s"
-                    % (METRIC1, METRIC2, PROJECT_ID)),
-            fail_ok=True, merge_stderr=True)
-        self.assertFirstLineStartsWith(
-            result.splitlines(), "Alarm with name='alarm1' exists (HTTP 409)")
-
         # CREATE FAIL MISSING PARAM
         self.assertRaises(
             exceptions.CommandFailed,
@@ -726,3 +760,135 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
             self.assertEqual(sorted(output_colums), sorted(alarm_list.keys()))
         self.assertNotIn(ALARM_ID,
                          [r['alarm_id'] for r in self.parser.listing(result)])
+
+    def test_update_gnresthr_gnaggrresthr(self):
+
+        RESOURCE_ID = uuidutils.generate_uuid()
+        # CREATE
+        result = self.aodh(u'alarm',
+                           params=(u"create "
+                                   "--type gnocchi_resources_threshold "
+                                   "--name alarm_gn123 --metric cpu_util "
+                                   "--resource-id %s --threshold 80 "
+                                   "--resource-type generic "
+                                   "--aggregation-method last "
+                                   % RESOURCE_ID))
+        alarm = self.details_multiple(result)[0]
+        ALARM_ID = alarm['alarm_id']
+        self.assertEqual('alarm_gn123', alarm['name'])
+        self.assertEqual('cpu_util', alarm['metric'])
+        self.assertEqual('80.0', alarm['threshold'])
+        self.assertEqual('last', alarm['aggregation_method'])
+        self.assertEqual('generic', alarm['resource_type'])
+
+        # UPDATE TYPE TO GNOCCHI_AGGREGATION_BY_RESOURCES_THRESHOLD
+        result = self.aodh(
+            'alarm', params=("update %s --type "
+                             "gnocchi_aggregation_by_resources_threshold "
+                             "--metric cpu --threshold 90 "
+                             "--query "
+                             '\'{"=": {"creator": "cr3at0r"}}\' '
+                             "--resource-type generic "
+                             "--aggregation-method last "
+                             % ALARM_ID))
+        alarm_updated = self.details_multiple(result)[0]
+        self.assertEqual(ALARM_ID, alarm_updated["alarm_id"])
+        self.assertEqual('cpu', alarm_updated['metric'])
+        self.assertEqual('90.0', alarm_updated['threshold'])
+        self.assertEqual('last', alarm_updated['aggregation_method'])
+        self.assertEqual('generic', alarm_updated['resource_type'])
+        self.assertEqual('{"=": {"creator": "cr3at0r"}}',
+                         alarm_updated['query'])
+        self.assertEqual('gnocchi_aggregation_by_resources_threshold',
+                         alarm_updated['type'])
+
+        # UPDATE TYPE TO GNOCCHI_RESOURCES_THRESHOLD
+        result = self.aodh(
+            'alarm', params=("update %s "
+                             "--type gnocchi_resources_threshold "
+                             "--metric cpu_util "
+                             "--resource-id %s --threshold 80 "
+                             "--resource-type generic "
+                             "--aggregation-method last "
+                             % (ALARM_ID, RESOURCE_ID)))
+        alarm_updated = self.details_multiple(result)[0]
+        self.assertEqual(ALARM_ID, alarm_updated["alarm_id"])
+        self.assertEqual('cpu_util', alarm_updated['metric'])
+        self.assertEqual('80.0', alarm_updated['threshold'])
+        self.assertEqual('last', alarm_updated['aggregation_method'])
+        self.assertEqual('generic', alarm_updated['resource_type'])
+        self.assertEqual('gnocchi_resources_threshold',
+                         alarm_updated['type'])
+
+        # DELETE
+        result = self.aodh('alarm', params="delete %s" % ALARM_ID)
+        self.assertEqual("", result)
+
+    def test_update_gnaggrresthr_gnaggrmetricthr(self):
+
+        METRIC1 = 'cpu'
+        METRIC2 = 'cpu_util'
+
+        # CREATE
+        result = self.aodh(
+            u'alarm',
+            params=(u"create "
+                    "--type "
+                    "gnocchi_aggregation_by_resources_threshold "
+                    "--name alarm123 --metric cpu --threshold 80 "
+                    "--query "
+                    '\'{"=": {"creator": "cr3at0r"}}\' '
+                    "--resource-type generic "
+                    "--aggregation-method last "))
+        alarm = self.details_multiple(result)[0]
+        ALARM_ID = alarm['alarm_id']
+        self.assertEqual('alarm123', alarm['name'])
+        self.assertEqual('cpu', alarm['metric'])
+        self.assertEqual('80.0', alarm['threshold'])
+        self.assertEqual('last', alarm['aggregation_method'])
+        self.assertEqual('generic', alarm['resource_type'])
+        self.assertEqual('{"=": {"creator": "cr3at0r"}}',
+                         alarm['query'])
+
+        # UPDATE TYPE TO GNOCCHI_AGGREGATION_BY_METRICS_THRESHOLD
+        result = self.aodh(
+            'alarm', params=("update %s --type "
+                             "gnocchi_aggregation_by_metrics_threshold "
+                             "--metrics %s "
+                             "--metrics %s "
+                             "--threshold 80 "
+                             "--aggregation-method last"
+                             % (ALARM_ID, METRIC1, METRIC2)))
+        alarm_updated = self.details_multiple(result)[0]
+        self.assertEqual(ALARM_ID, alarm_updated["alarm_id"])
+        metrics = "[u'cpu', u'cpu_util']" if six.PY2 else "['cpu', 'cpu_util']"
+        self.assertEqual(metrics, alarm_updated['metrics'])
+        self.assertEqual('80.0', alarm_updated['threshold'])
+        self.assertEqual('last', alarm_updated['aggregation_method'])
+        self.assertEqual('gnocchi_aggregation_by_metrics_threshold',
+                         alarm_updated['type'])
+
+        # UPDATE TYPE TO GNOCCHI_AGGREGATION_BY_RESOURCES_THRESHOLD
+        result = self.aodh(
+            'alarm', params=("update %s --type "
+                             "gnocchi_aggregation_by_resources_threshold "
+                             "--metric cpu --threshold 80 "
+                             "--query "
+                             '\'{"=": {"creator": "cr3at0r"}}\' '
+                             "--resource-type generic "
+                             "--aggregation-method last "
+                             % ALARM_ID))
+        alarm_updated = self.details_multiple(result)[0]
+        self.assertEqual(ALARM_ID, alarm_updated["alarm_id"])
+        self.assertEqual('cpu', alarm_updated['metric'])
+        self.assertEqual('80.0', alarm_updated['threshold'])
+        self.assertEqual('last', alarm_updated['aggregation_method'])
+        self.assertEqual('generic', alarm_updated['resource_type'])
+        self.assertEqual('{"=": {"creator": "cr3at0r"}}',
+                         alarm_updated['query'])
+        self.assertEqual('gnocchi_aggregation_by_resources_threshold',
+                         alarm_updated['type'])
+
+        # DELETE
+        result = self.aodh('alarm', params="delete %s" % ALARM_ID)
+        self.assertEqual("", result)
